# -*- coding: utf-8 -*-
__author__ = 'Dan Tracy'
__email__ = 'djt5019 at gmail dot com'

import os
import datetime
import sqlite3
import atexit
import logging

import utils

from episode import Episode, Special
from constants import RESOURCE_PATH
from settings import Settings

if not utils.file_exists_in_resources(Settings['sql']):
    utils.create_default_sql_schema()

with utils.open_file_in_resources(Settings['sql']) as sql:
    schema = sql.read()


class Cache(object):
    """ Our database logic class"""
    def __init__(self, dbName=u"episodes.db"):
        """Establish a connection to the show database"""
        if not dbName or not isinstance(dbName, basestring):
            raise ValueError("Empty database name passed to cache")

        if dbName != ':memory:':
            dbName = os.path.join(RESOURCE_PATH, dbName)

        try:
            if not os.path.exists(dbName):
                self.connection = sqlite3.connect(dbName, detect_types=sqlite3.PARSE_DECLTYPES)
                logging.debug("Creating database: {}".format(dbName))
                self.connection.executescript(schema)
            else:
                self.connection = sqlite3.connect(dbName, detect_types=sqlite3.PARSE_DECLTYPES)
        except sqlite3.OperationalError as e:
            logging.error("Error connecting to database: {}".format(e))
            self.connection = None

        if self.connection:
            self.cursor = self.connection.cursor()

            #Make sure everything is utf-8
            self.connection.text_factory = lambda x: unicode(x, 'utf-8')
            atexit.register(self.close)
        else:
            logging.critical("Unable to open a connection to the database")
            raise sqlite3.OperationalError

    def close(self):
        """ Commits any changes to the database then closes connections to it"""
        self.cursor.close()
        self.connection.commit()
        self.connection.close()
        logging.info("Connections have been closed")

    def add_show(self, showTitle, episodes):
        """ If we find a show on the internet that is not in our database
        we can use this function to add it into our database for the future"""
        if not (showTitle and episodes):
            raise ValueError("Empty show title or specials list passed to add_specials")

        if not (isinstance(showTitle, basestring) and isinstance(episodes, list)):
            raise ValueError("add_show expects a string and a list")

        title = showTitle
        time = datetime.datetime.now()

        self.cursor.execute("INSERT INTO shows values (NULL, ?, ?)", (title, time))

        showId = self.cursor.lastrowid

        for eps in episodes:
            show = (showId, eps.title, eps.season, eps.episode_number,)
            self.cursor.execute(
                "INSERT INTO episodes values (NULL, ?, ?, ?, ?)", show)

    def remove_show(self, sid):
        """Removes show and episodes matching the show id """
        if not isinstance(sid, int):
            raise ValueError("remove_show expects an integer primary key")

        self.cursor.execute("DELETE FROM episodes where sid=?", (sid,))
        self.cursor.execute("DELETE FROM specials where sid=?", (sid,))
        self.cursor.execute("DELETE FROM shows where sid=?", (sid,))

    def get_episodes(self, showTitle):
        """Returns the episodes associated with the show id"""
        if not showTitle:
            raise ValueError("get_episodes expects a string")

        title = (showTitle, )
        self.cursor.execute(
            """SELECT e.eptitle, e.showNumber,e.season, s.sid, s.time
               FROM episodes AS e INNER JOIN shows AS s
               ON s.sid=e.sid AND s.title=?""", title)

        result = self.cursor.fetchall()
        eps = []

        if not result:
            return eps

        diffDays = (datetime.datetime.now() - result[0][-1])

        logging.info("{} days old".format(diffDays.days))

        update_days = Settings['db_update']

        if diffDays.days >= update_days:
            #If the show is older than a week remove it then return not found
            logging.warning("Show is older than a week, removing...")
            sid = result[0][-2]
            self.remove_show(sid)
            return eps

        for count, episode in enumerate(result, start=1):
            eps.append(Episode(episode[0], episode[1], episode[2], count))

        return eps

    def get_specials(self, showTitle):
        """ Returns a list of Special episode objects """
        if not showTitle:
            raise ValueError("Show title was not an acceptable type: requires string")

        title = (showTitle, )
        self.cursor.execute(
            """SELECT sp.title, sp.showNumber, sp.type
               FROM specials AS sp INNER JOIN shows
               ON sp.sid=shows.sid AND shows.title=?""", title)

        result = self.cursor.fetchall()

        if result is not None:
            eps = [Special(*episode) for episode in result]

        return eps

    def add_specials(self, showTitle, episodes):
        """ Add a list of special episode objects """
        query = """
        INSERT INTO specials (mid,  sid, title, showNumber, type)
        SELECT NULL, sid, ?, ?, ?
        FROM shows
        WHERE shows.title=?
        """

        if not (showTitle and episodes):
            return

        if not (isinstance(showTitle, basestring) and isinstance(episodes, list)):
            raise ValueError("add_specials expects a string and a list")

        for eps in episodes:
            show = (eps.title, eps.num, eps.type, showTitle)
            self.cursor.execute(query, show)

    def recreate_cache(self):
        self.cursor.executescript(schema)



# -*- coding: utf-8 -*-
__author__ = 'Dan Tracy'
__email__ = 'djt5019 at gmail dot com'

import sys
import logging

from os import listdir
from itertools import ifilter

import constants

from episode import Special


def locate_show(title):
    """Polls the web sources looking for the show"""
    modules = []
    eps = []
    specials = []

    sys.path.append(constants.WEB_SOURCES_PATH)
    # This will import all the modules within the web_sources directory so that
    # we can easily plug in new sources for finding episode information so long
    # as they define a poll function

    for m in listdir(constants.WEB_SOURCES_PATH):
        if m.endswith('.py') and not m.startswith('__'):
            logging.info("Importing web resource {}".format(m[:-3]))

            x = __import__(m[:-3])
            if not hasattr(x, 'poll'):
                logging.error("Module {} doesn't have a poll method defined, ignoring module".format(m))
                continue

            if not hasattr(x, 'priority'):
                logging.error("Module {} doesn't have a priority defined, defaulting to 0".format(m))
                x.priority = 0

            modules.append(x)

    # If the modules have a poll priority we will respect it
    # by sorting the list by priority, higher number have higher precedence
    modules = sorted(modules, key=lambda x: x.priority, reverse=True)

    logging.info("Searching for {}".format(title))

    episodes = constants.SHOW_NOT_FOUND

    for source in modules:
        logging.info("Polling {0}".format(source.__name__))

        episodes = source.poll(title)

        if episodes:
            logging.info("LOCATED {0}".format(title))
            break

        logging.info("Unable to locate {0} at {1}".format(title, source.__name__))

    if not episodes:
        logging.info("Unable to locate the show: " + title)
        return (eps, specials)

    for e in episodes:
        if isinstance(e, Special):
            specials.append(e)
        else:
            eps.append(e)

    eps = sorted(ifilter(lambda x: x.episode_number > 0, eps), key=lambda z: z.episode_count)
    specials = sorted(ifilter(lambda x: x.num > 0, specials), key=lambda z: z.num)

    return (eps, specials)



# -*- coding: utf-8 -*-
__author__ = 'Dan Tracy'
__email__ = 'djt5019 at gmail dot com'

import unittest

from episode_renamer import poll_sources


class TestPollSources(unittest.TestCase):
    def test(self):
        assert(poll_sources is not None)



# -*- coding: utf-8 -*-
__author__ = 'Dan Tracy'
__email__ = 'djt5019 at gmail dot com'

import logging
import logging.config

from os.path import join
from datetime import datetime

from constants import RESOURCE_PATH
from settings import Settings


def init_logging():
    logging.config.dictConfig(log_config)
    logging.debug("APPLICATION START: {}".format(datetime.now()))


def shutdown_logging():
    """
    Properly shutdown the loggers called upon program termination,
    registered to atexit in __init__.py
    """
    logging.debug("APPLICATION END: {}".format(datetime.now()))
    logging.shutdown()


log_config = {
    'version': 1,
    'formatters': {
        'console_format': {
            'format': '%(levelname)s | "%(message)s"'
        },
        'file_format': {
            'format': '%(levelname)s | %(module)s.%(funcName)s - "%(message)s"'
        }
    },
    'handlers': {
        'file': {
            'level': 'DEBUG',
            'class': 'logging.handlers.RotatingFileHandler',
            'formatter': 'file_format',
            'filename': r'{}'.format(join(RESOURCE_PATH, Settings['log_file'])),
            'maxBytes': 2 ** 20,
            'backupCount': 3
        },
        'console': {
            'level': 'WARNING',
            'class': 'logging.StreamHandler',
            'formatter': 'console_format',
        }
    },
    'root': {
        'handlers': ['console', 'file'],
        'qualname': 'root',
        'level': 'DEBUG'
    }
}



# -*- coding: utf-8 -*-
#!/usr/bin/env python

from __future__ import unicode_literals

__author__ = 'Dan Tracy'
__email__ = 'djt5019 at gmail dot com'

import os
import logging

from PySide import QtGui, QtCore

from episode_renamer.Parser import Parser
from episode_renamer.Episode import Show, EpisodeFormatter
from episode_renamer.Cache import Cache
from episode_renamer.Settings import Settings

from episode_renamer import Utils

cache = Cache()
parser = Parser(cache=cache)
Utils.load_renamed_file()

Settings['title'] = ""

## Snuff the console output for the gui
log = logging.getLogger()
for handler in log.handlers:
    if isinstance(handler, logging.StreamHandler):
        handler.setLevel(logging.CRITICAL)
        del log
        break


class UpdateDisplaySignal(QtCore.QObject):
    update_directory = QtCore.Signal()
    update_episodes = QtCore.Signal()


class Window(QtGui.QMainWindow):
    def __init__(self, parent=None):
        super(Window, self).__init__(parent)
        self.form = Form()

        self.setCentralWidget(self.form)

        exitAction = QtGui.QAction('&Exit', self)
        exitAction.setShortcut("&Ctrl+Q")
        exitAction.setStatusTip("&Exit the application")
        exitAction.triggered.connect(self.close)

        undoRenameAction = QtGui.QAction("&Undo Rename", self)
        undoRenameAction.setStatusTip("&Show the undo rename dialog")
        undoRenameAction.triggered.connect(self.form.displayUndoRenameDialog)

        helpAction = QtGui.QAction('&Info', self)
        helpAction.setShortcut('F1')
        helpAction.setStatusTip("Show help menu")
        helpAction.triggered.connect(self.printHelp)

        menubar = self.menuBar()
        fileMenu = menubar.addMenu('&File')
        helpMenu = menubar.addMenu('&Help')

        fileMenu.addAction(undoRenameAction)
        fileMenu.addAction(exitAction)
        helpMenu.addAction(helpAction)

        self.setWindowTitle('Menubar')

    def printHelp(self):
        QtGui.QMessageBox.information(self, "Help Dialog", "Help Text")


class Form(QtGui.QWidget):
    def __init__(self, parent=None):
        super(Form, self).__init__(parent)

        self.epLine = QtGui.QLineEdit()
        self.seasonBox = QtGui.QComboBox(self)
        self.findShowBtn = QtGui.QPushButton("&Find Show")
        self.epList = QtGui.QListWidget()
        self.fmtLine = QtGui.QLineEdit()

        self.seasonBox.addItem("All")

        self.findDirBtn = QtGui.QPushButton("&Find Dir")
        self.dirList = QtGui.QListWidget()
        self.renameBtn = QtGui.QPushButton("&Rename")

        # Connect our signals
        self.epLine.returnPressed.connect(self.findShow)
        self.findDirBtn.clicked.connect(self.displayDirDialog)
        self.findShowBtn.clicked.connect(self.findShow)
        self.renameBtn.clicked.connect(self.displayRenameDialog)
        self.fmtLine.editingFinished.connect(self.updateFormat)
        self.seasonBox.activated[str].connect(self.filterSeasons)

        # Prepare the display update signals
        self.updater = UpdateDisplaySignal()
        self.updater.update_directory.connect(self.updateDirectoryListing)
        self.updater.update_episodes.connect(self.updateEpisodeListing)

        #Set the left layout
        leftWidget = QtGui.QWidget()
        stackedWidget = QtGui.QWidget()
        topLayout = QtGui.QVBoxLayout()
        vertLayout = QtGui.QFormLayout()
        vertLayout.addRow("&Search for show", self.epLine)
        vertLayout.addRow("&Season", self.seasonBox)
        vertLayout.addRow("&Episode Format", self.fmtLine)
        stackedWidget.setLayout(vertLayout)
        topLayout.addWidget(stackedWidget)
        topLayout.addWidget(self.epList)
        topLayout.addWidget(self.findShowBtn)
        leftWidget.setLayout(topLayout)

        #Set the right layout
        label = QtGui.QLabel("Current Directory")
        self.currentDirLabel = QtGui.QLabel("<No Directory Selected>")
        label.setBuddy(self.currentDirLabel)
        rightWidget = QtGui.QWidget()
        rightLayout = QtGui.QVBoxLayout()
        rightLayout.addWidget(label)
        rightLayout.addWidget(self.currentDirLabel)
        rightLayout.addWidget(self.findDirBtn)
        rightLayout.addWidget(self.dirList)
        rightLayout.addWidget(self.renameBtn)
        rightWidget.setLayout(rightLayout)

        #Connect the left widget and right widget
        displayBox = QtGui.QHBoxLayout()
        displayBox.addWidget(leftWidget)
        displayBox.addWidget(rightWidget)

        self.setLayout(displayBox)
        self.show = Show("")
        self.renameDir = ""
        self.episodes = []
        self.formatter = EpisodeFormatter(self.show)
        self.show.formatter = self.formatter
        self.fmtLine.setText(self.show.formatter.format_string)

    def updateDirectoryListing(self):
        self.epLine.setText(os.path.split(self.renameDir)[1])
        self.currentDirLabel.setText(os.path.abspath(self.renameDir))
        self.dirList.clear()
        self.epList.clear()
        for ep in Utils.clean_filenames(self.renameDir):
            self.dirList.addItem(ep.name)

    def updateEpisodeListing(self):
        if not self.episodes:
            InfoMessage(self, "Find Show", "Unable to find show, check spelling and try again")
            return

        self.epList.clear()
        for ep in self.episodes:
            self.epList.addItem(self.show.formatter.display(ep))

    def filterSeasons(self, text):
        if text == 'All':
            self.updater.update_episodes.emit()
            return

        season = int(text)
        self.episodes = self.show.get_season(season)

        self.updater.update_episodes.emit()

    def updateFormat(self):
        if self.fmtLine.text() != '':
            self.formatter.format_string = self.fmtLine.text()

        if self.show.title != "":
            self.updater.update_episodes.emit()

    def findShow(self):
        showTitle = Utils.remove_punctuation(self.epLine.text().strip())

        if not showTitle:
            InfoMessage(self, "Find Show", "No show specified")
            return

        Settings['title'] = showTitle

        parser.setShow(showTitle)
        self.show = parser.getShow()
        self.formatter.show = self.show
        self.show.formatter = self.formatter
        self.episodes = self.show.episodes
        self.seasonBox.clear()
        self.seasonBox.addItem("All")
        if self.episodes:
            for s in xrange(self.show.num_seasons):
                self.seasonBox.addItem(str(s + 1))

        self.updater.update_episodes.emit()

    def displayDirDialog(self):
        newDir = QtGui.QFileDialog.getExistingDirectory(self, 'Choose Directory', r'G:\TV\Misc')
        if newDir == '':
            return

        self.renameDir = newDir
        self.dirList.clear()

        self.updater.update_directory.emit()

    def displayRenameDialog(self):
        if self.renameDir == "":
            InfoMessage(self, "Rename Files", "No Directory Selected")
            return

        if self.epList.count() == 0:
            InfoMessage(self, "Rename Files", "No Show Information Retrieved")
            return

        files = Utils.prepare_filenames(self.renameDir, self.show)
        dialog = RenameDialog(files, self)
        dialog.finished.connect(self.updateDirectoryListing)
        dialog.exec_()

        self.updater.update_directory.emit()

    def displayUndoRenameDialog(self):
        items = []

        for d in Settings['backup_list']:
            items.append(Settings['backup_list'][d]['name'])

        text, ok = QtGui.QInputDialog.getItem(self, "Select show", "", items, editable=False)

        if not ok or not text:
            return

        items = []
        for d in Settings['backup_list']:
            renamed_entry = Settings['backup_list'][d]
            if text == renamed_entry['name']:
                print d
                self.renameDir = d
                items = renamed_entry['file_list']
                break

        dialog = RenameDialog(items, self)
        dialog.finished.connect(self.updateDirectoryListing)
        dialog.exec_()

        self.updater.update_directory.emit()


class InfoMessage(object):
    def __init__(self, parent, windowTitle, msg):
        QtGui.QMessageBox.information(parent, windowTitle, msg)


class RenameDialog(QtGui.QDialog, object):
    def __init__(self, files, parent=None):
        super(RenameDialog, self).__init__(parent)
        self.setGeometry(100, 100, 750, 500)
        self.setWindowTitle("Rename Files Utility")

        self.files = files

        self.buttonBox = QtGui.QDialogButtonBox(QtCore.Qt.Vertical)
        self.buttonBox = QtGui.QDialogButtonBox(QtGui.QDialogButtonBox.Cancel | QtGui.QDialogButtonBox.Ok)
        self.buttonBox.rejected.connect(self.close)

        self.model = QtGui.QStandardItemModel()
        self.view = QtGui.QListView()

        self.view.setModel(self.model)

        layout = QtGui.QVBoxLayout()
        layout.addWidget(self.view)
        layout.addWidget(self.buttonBox)

        self.setLayout(layout)

        if not self.files:
            self.model.appendRow(QtGui.QStandardItem("No files need to be renamed"))
            self.buttonBox.accepted.connect(self.close)
            self.buttonBox.accepted.connect(self.close)
        else:
            self.buttonBox.accepted.connect(self.rename)
            for old, new in self.files:
                item = QtGui.QStandardItem()

                old_name = Utils.encode(os.path.split(old)[1])
                new_name = Utils.encode(os.path.split(new)[1])
                string = Utils.encode("OLD: {}\nNEW: {}\n".format(old_name, new_name))
                item.setText(string)
                item.setCheckState(QtCore.Qt.Checked)
                item.setCheckable(True)
                item.rename_info = (old, new)

                if item.text():
                    self.model.appendRow(item)

        self.view.show()
        self.show()

    def rename(self):
        files = []

        for itemIndex in xrange(self.model.rowCount()):
            item = self.model.item(itemIndex)
            if item.checkState() == QtCore.Qt.Checked:
                files.append(item.rename_info)

        old_order, errors = Utils.rename(files, 'yes')
        self.model.clear()
        self.buttonBox.accepted.disconnect()

        Utils.save_renamed_file_info(old_order, Settings['title'])

        if errors:
            self.model.appendRow(QtGui.QStandardItem("&The following files could not be renamed\n"))
            for e in errors:
                self.model.appendRow(QtGui.QStandardItem(e[0]))
                logging.error('Unable to rename: {}'.format(e))
        else:
            self.model.appendRow(QtGui.QStandardItem("The files were renamed successfully"))
            logging.info("Filenames renamed succesfully")

        self.buttonBox.accepted.connect(self.close)
        self.buttonBox.rejected.connect(self.close)


def main():
    app = QtGui.QApplication("&My App")
    form = Window()
    form.show()

    ret = app.exec_()
    form.close()

    return ret

if __name__ == '__main__':
    main()




# -*- coding: utf-8 -*-
__author__ = 'Dan Tracy'
__email__ = 'djt5019 at gmail dot com'

import os
import sys
import re
import time
import json
import logging

import constants
import episode
from tempfile import TemporaryFile

from settings import Settings

import requests
import requests.exceptions


def get_url_descriptor(url):
    """
    Returns an url descriptor or None on failure
    """
    try:
        resp = requests.get(url)
    except requests.exceptions.ConnectionError:
        logging.error("Error connecting to {}".format(url))
        return None

    if resp.ok:
        return resp

    return None


def is_valid_file(filename):
    """
    Returns true if the filename is a valid video file
    """
    ext = os.path.splitext(filename)[1].lower()

    return os.path.isfile(filename) and ext in constants.VIDEO_EXTENSIONS


##############################
## Renaming utility functions
##############################
def regex_search(filename):
    """
    Compare the filename to each of the regular expressions for a match
    """

    # deal with anything in brackets ourselves, they tend to throw off the regexes
    info_dict = {}
    checksum = constants.checksum_regex.search(filename)
    filename = constants.checksum_regex.sub("", filename)
    season = constants.bracket_season_regex.search(filename)

    filename = constants.remove_junk_regex.sub("", filename)

    result = None
    for count, regex in enumerate(constants.regexList):
        result = regex.search(filename)
        if result:
            logging.info("Regex #{} matched {}".format(count, filename.encode(Settings['encoding'], 'ignore')))
            break

    if not result and not season:
        msg = "Unable to find information on: {}".format(filename)
        logging.error(msg)
        raise Exception(msg)

    # Work with the result dict rather than the annoying groupdicts
    result = result.groupdict() if result else {}
    result.update(**checksum.groupdict() if checksum else {})
    result.update(**season.groupdict() if season else {})

    logging.info(result)

    if 'junk' in result:
        return info_dict

    info_dict['checksum'] = int(result.get('sum', '0x0'), base=16)

    if 'special' in result:
        info_dict['special_number'] = int(result['special'])
        info_dict['special_type'] = result.get('type', 'OVA')
    else:
        info_dict['episode_number'] = int(result['episode'])
        info_dict['season'] = int(result.get('season', '-1'))

    return info_dict


def clean_filenames(path):
    """
    Attempts to extract order information about the files passed
    returns: list of EpisodeFiles
    """
    # Filter out anything that doesnt have the correct extension and
    # filter out any directories
    files = []
    for f in os.listdir(path):
        if is_valid_file(os.path.join(path, f)):
            files.append(f)
        else:
            logging.info("Invalid file: {}".format(f))

    if not files:
        logging.error("No video files were found in {}".format(path))
        return []

    cleanFiles = []
    # We are going to store the episode number and path in a tuple then sort on the
    # episodes number.  Special episodes will be appended to the end of the clean list
    for f in files:
        info = regex_search(f)

        if info:
            cleanFiles.append(episode.EpisodeFile(os.path.join(path, f), **info))

    if not cleanFiles:
        logging.error("The files could not be matched")
        return cleanFiles

    logging.info("Successfully cleaned the file names")

    return cleanFiles


def prepare_filenames(path, show):
    """
    Rename the files located in 'path' to those in the list 'show', modifies
    the show objects episodeList/specialsList
    """
    path = os.path.abspath(path)
    sameCount = 0
    cleanFiles = []
    files = clean_filenames(path)
    #Match the list of EpisodeFiles to the list of shows in the 'show' variable
    if not files:
        logging.info("No files were able to be renamed")
        return

    for f in files:
        if f.is_ova:
            episode = show.get_special(f.special_number)
        elif f.episode_number > show.max_episode_number:
            episode = show.get_special(f.episode_number - show.max_episode_number)
        else:
            episode = show.get_episode(season=f.season, episode=f.episode_number)

        if not episode:
            logging.warning("Could not find an episode for {}".format(f.name))
            continue

        # attach the episode file to the corresponding episode entry
        episode.episode_file = f

        fileName = encode(f.name)
        newName = replace_invalid_path_chars(show.formatter.display(episode) + f.ext)

        if newName == fileName:
            logging.info("File {} and Episode {} have same name".format(f.name, episode.title))
            sameCount += 1
            continue

        newName = trim_long_filename(os.path.join(path, newName))

        episode.episode_file.new_name = newName
        cleanFiles.append((f.path, newName))

    if sameCount > 0:
        msg = "1 file" if sameCount == 1 else "{} files".format(sameCount)
        logging.warning(
            "{} in this directory would have been renamed to the same filename".format(msg))

    return cleanFiles


def rename(files, resp=""):
    """
    Performs the actual renaming of the files
    returns a tuple:
    (list of tuples with old info, list of file that cold not be renamed)
    """
    errors = []
    if resp == '':
        resp = raw_input("\nDo you wish to rename these files [y|N]: ").lower()

    if not resp.startswith('y'):
        logging.info("Changes were not committed to the files")
        return errors

    old_order = []

    for old, new in files:
        try:
            os.rename(old, new)
            old_order.append((new, old))
        except OSError as e:
            errors.append((old, e))

    return old_order, errors

#############################
## Undo rename functionality
#############################
## data format, save as json:
## { path_name_str_1: {
##      show_name: string,
##      num_files: int,
##      file_list: [ (new_name, old_name)]
##      },
##  path_name_str_2: {
##      show_name: string,
##      num_files: int,
##      file_list: [ (new_name, old_name)]
##      }, ...
## }
#############################


def save_renamed_file_info(old_order, show_title=None):
    """
    Save the previous names from the last renaming operation to disk
    """
    logging.info("Backing up old filenames")
    path = os.path.split(old_order[0][0])[0]

    # If we have a title provided by the comand line, use that otherwise fall
    # fall back on the folder name (at least it's something)
    if show_title:
        name = show_title
    else:
        name = os.path.split(path)[1]

    fmt = dict(num_files=len(old_order), file_list=old_order, name=name)
    Settings['backup_list'][path] = fmt

    with open_file_in_resources(Settings['rename_backup'], 'w') as f:
        json.dump(Settings['backup_list'], f)


def find_old_filenames(path, show_title=None):
    """
    Returns a dict with the filenames and the number of files
    """
    logging.info("Loading up old filenames")
    if 'backup_list' not in Settings:
        load_renamed_file()

    if not show_title:
        return Settings['backup_list'].get(path, dict(name="", file_list=[], num_files=0))

    try:
        return Settings['backup_list']['path']
    except KeyError:
        for d in Settings['backup_list']:
            info = Settings['backup_list'][d]
            if info['name'] == show_title:
                return info

    return {}


def load_renamed_file():
    """
    Loads the json file of renamed shows into Settings['backup_list']
    """
    logging.info("Loading the renamed episode json file")
    if not file_exists_in_resources(Settings['rename_backup']):
        logging.warn("Json file [{}] with old information could not be found".format(Settings['rename_backup']))
        Settings['backup_list'] = {}
    else:
        with open_file_in_resources(Settings['rename_backup']) as f:
            try:
                Settings['backup_list'] = json.load(f)
            except ValueError:
                logging.warning("The json file is empty")
                Settings['backup_list'] = {}


########################
## Text based functions
#######################

def trim_long_filename(name):
    if len(name) > 255:
        ext = os.path.splitext(name)[1]
        logging.error('The filename "{}" may be too long to rename, truncating'.format(name))
        offset = 255 - len(ext)
        name = name[:offset] + ext
    return name


def remove_punctuation(title):
    """
    Remove any punctuation and whitespace from the title
    """
    name, ext = os.path.splitext(title)
    exclude = set('!"#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~')
    name = ''.join(ch for ch in name if ch not in exclude)
    return name + ext


def replace_invalid_path_chars(path, replacement='-'):
    """
    Replace invalid path character with a different, acceptable, character
    """
    exclude = set('\\/"?<>|*:')
    path = ''.join(ch if ch not in exclude else replacement for ch in path)
    return path


if sys.platform != 'win32':
    trim_long_filename = lambda name: name


def prepare_title(title):
    """
    Remove any punctuation and whitespace from the title
    """

    if not title:
        return ""

    title = remove_punctuation(title).split()

    if not title:
        return ""

    if title[0].lower() == 'the':
        title.remove(title[0])

    out = []
    for n in title:
        try:
            value = num_to_text(int(n))
            out.append(value)
        except Exception:
            out.append(n)

    return ''.join(out)


def num_to_text(num):
    """
    Converts a number up to 999 to it's English representation
    """
    # The purpose of this function is to resolve numbers to text so we don't
    # have additional entries in the database for the same show.  For example
    # The 12 kingdoms and twelve kingdoms will yield the same result in the DB

    if num < 20:
        return constants.NUM_DICT[str(num)]

    args = []
    num = str(num)

    while num:
        digit = int(num[0])
        length = len(num)

        if length == 3:
            args.append(constants.NUM_DICT[num[0]])
            args.append("hundred")
        else:
            value = str(digit * (10 ** (length - 1)))
            args.append(constants.NUM_DICT[value])

        num = num[1:]

    return '_'.join(args)


def encode(text, encoding='utf-8'):
    """
    Returns a unicode representation of the string
    """
    if isinstance(text, basestring):
        if not isinstance(text, unicode):
            text = unicode(text, encoding, 'ignore')
    return text


###############################
##  Web Source Functionality
###############################

show_not_found = constants.SHOW_NOT_FOUND


def able_to_poll(site, delay=None, wait=False):
    """
    Prevents flooding by waiting two seconds from the last poll
    """
    if not delay:
        delay = Settings['poll_delay']

    if not Settings['access_dict']:
        Settings['access_dict'] = load_last_access_times()

    last_access = Settings['access_dict'].get(site, -1)
    now = int(time.time())

    flooding = True

    if (last_access < 0) or (now - last_access >= delay):
        Settings['access_dict'][site] = now
        flooding = False

    if flooding and wait:
        logging.warn('Possible flooding of "{}" detected"'.format(site))
        logging.warn("Waiting for {} second".format(delay))
        time.sleep(int(delay))
        flooding = False

    elif flooding:
        logging.warn('Possible flooding of "{}" detected'.format(site))

    return not flooding


def open_file_in_resources(name, mode='r'):
    """
    Returns a file object if the filename exists in the resources directory
    """
    name = os.path.split(name)[1]
    name = os.path.join(constants.RESOURCE_PATH, name)

    if mode in ('w', 'wb'):
        return open(name, mode)

    if file_exists_in_resources(name):
        return open(name, mode)

    raise OSError("File {} was not found".format(name))


def file_exists_in_resources(name):
    """
    Returns true if the filename exists in the resources directory
    """
    name = os.path.split(name)[1]
    name = os.path.join(constants.RESOURCE_PATH, name)
    return os.path.exists(name)


def regex_compile(regex):
    """
    Returns a compiled regex instance, ignore case and verbose are on by default
    """
    return re.compile(regex, re.I | re.X)


def regex_sub(pattern, replacement, target):
    """
    Substitutes the replacement in the target if it matches the pattern
    """
    return re.sub(pattern, replacement, target)


def temporary_file(suffix):
    """
    Returns a file object to a temporary file
    """
    return TemporaryFile(suffix=suffix)


def save_last_access_times():
    """
    Save the last access times dictionary to a file in resources
    """
    if not Settings['access_dict']:
        return False

    with open_file_in_resources(Settings['access_time_file'], 'w') as p:
        json.dump(Settings['access_dict'], p)

    return True


def load_last_access_times():
    """
    Load the access times dictionary from the file in resource path
    """
    if not file_exists_in_resources(Settings['access_time_file']):
        return {}

    with open_file_in_resources(Settings['access_time_file']) as f:
        return json.load(f)


#################
## Creation of defualt functions/files functionality
#################

def initalize_resource_folder():
    print "[+] Creating resource path"
    print "[+] Path = {}".format(constants.RESOURCE_PATH)
    os.makedirs(constants.RESOURCE_PATH, 0755)
    print "[+] Updating the AniDb database file"
    utils.update_db()
    print "[+] Creating a renamed file backup listing"
    utils.create_new_backup_file()
    print "[+] Creating new sql schema file"
    utils.create_default_sql_schema()



def create_new_backup_file():
    logging.info("Creating a new rename info backup file")
    with open_file_in_resources(Settings['rename_backup'], 'w') as f:
        json.dump({}, f)


def update_db():
    logging.info("Attempting to update AniDb database file")
    one_unix_day = 24 * 60 * 60

    def _download():
        with open_file_in_resources(Settings['anidb_db_file'], 'w') as f:
            logging.info("Retrieving AniDB Database file")
            url = get_url_descriptor(Settings['anidb_db_url'])

            f.write(url.content)
        logging.info("Successfully updated anidb_db_file")

    if not file_exists_in_resources(Settings['anidb_db_file']):
        _download()
    elif able_to_poll('db_download', one_unix_day):
        _download()
    else:
        logging.error("Attempting to download the database file multiple times")


def create_default_sql_schema():
    logging.info("Creating the default sql schema")
    with open_file_in_resources(Settings['sql'], 'w') as f:
        f.write(constants.CREATE_DB_QUERY)



# -*- coding: utf-8 -*-

__author__ = 'Dan Tracy'
__email__ = 'djt5019 at gmail dot com'

import shutil
import tempfile
import os
import time

from episode_renamer import utils

temp_dir = None


def create_temp_files(files):
    global temp_dir
    if not temp_dir:
        temp_dir = tempfile.mkdtemp()

        files = [open(os.path.join(temp_dir, f), 'w') for f in files]

    return files

def teardown():
    global temp_dir
    if temp_dir:
        print("Tearing down {}".format(temp_dir))
        shutil.rmtree(temp_dir)
        temp_dir = None


filenames = [
        "[group] series - 01[DEADBEEF].mkv",
        "[group]_series_-_02.mkv",
        "[group] series - S01E03.mkv",
        "[group] series - [01x04].mkv",
        "[group] series - episode 05.mkv",
        "[group] series - ep 06.mkv",
        "[group] series with spaces in title - 07.mkv",
        "[group] series with 1 in title - 08.mkv",
        "[group]   series with - 09 [DEADBEEF].mkv",
        "[group]series - 10 [h.264][720p][DEADBEEF].mkv",
        "series - season 1 - episode 11.mkv",
        "[group] series - DVD 12.mkv",
]


def test_get_url_descriptor():
    good_site = utils.get_url_descriptor("http://www.google.com")
    assert good_site != None
    assert good_site.ok == True

    bad_site = utils.get_url_descriptor("http://www.google.com/does_not_exist")
    assert bad_site == None


def test_is_valid_file():
    fake_files = [
    "C:\\Users\\SomeUser\\Files\\file1.mkv",
    "C:\\Users\\SomeUser\\Files\\file2.mov",
    "C:\\Users\\SomeUser\\Files\\file3.dvi",
    "/home/user/files/file3.avi",
    "/home/user/files/file4.mpeg"
    ]

    assert not all([utils.is_valid_file(f) for f in fake_files])

    temp_files = [
        utils.temporary_file(suffix=".mkv"),
        utils.temporary_file(suffix=".mov"),
        utils.temporary_file(suffix=".flv"),
        utils.temporary_file(suffix=".avi"),
        utils.temporary_file(suffix=".mpeg"),
    ]

    for f in temp_files:
        assert utils.is_valid_file(f.name)

    [f.close() for f in temp_files]


def test_regex_search():
    for episode, f in enumerate(filenames, 1):
        g = utils.regex_search(f)

        assert g is not None

        print f, g
        if 'sum' in g:
            assert g['sum'] == "DEADBEEF"

        if 'episode' in g:
            assert episode == int(g['episode'])

        if 'season' in g:
            assert int(g['season']) == 1

        if 'special' in g:
            assert g['special'] == "12"
            assert g['type'] == 'DVD'

test_regex_search.tags = ['regex', 'a']


def test_clean_filenames():
    create_temp_files(filenames)

    clean_files = utils.clean_filenames(temp_dir)

    assert clean_files is not None


def test_rename():
    create_temp_files(filenames)

    files = []
    for index, f in enumerate(os.listdir(temp_dir)):
        old = os.path.join(temp_dir, f)
        new = os.path.join(temp_dir, "{:02}.avi".format(index))
        files.append((old, new))

    assert utils.rename(files, 'y') == []

    for index, f in enumerate(os.listdir(temp_dir)):
        assert f == "{:02}.avi".format(index)

    files = utils.find_old_filenames(temp_dir)['file_list']

    assert utils.rename(files, 'y') == []

    for new in os.listdir(temp_dir):
        assert new in filenames

    teardown()


def test_remove_punctuation():
    string = "This-is_a/test\\string!@#$%$*&*(%)-=+_`~"
    expected = "Thisisateststring"

    assert utils.remove_punctuation(string) == expected

    string = "this is a test string"
    expected = "this is a test string"

    assert utils.remove_punctuation(string) == expected


def test_replace_invalid_path_chars():
    string = "file\\name?_<1>.exe"
    expected = "file-name-_-1-.exe"

    assert utils.replace_invalid_path_chars(string) == expected

    string = "file\\name?_<1>.exe"
    expected = "file=name=_=1=.exe"

    assert utils.replace_invalid_path_chars(string, replacement='=') == expected


def test_prepare_title():
    assert utils.prepare_title(None) == ""
    assert utils.prepare_title("Hawaii Five-O") == "HawaiiFiveO"
    assert utils.prepare_title("Breaking Bad") == "BreakingBad"
    assert utils.prepare_title("The Office!@#$%^&*()>_") == "Office"
    assert utils.prepare_title("The Big 10") == "Bigten"
    assert utils.prepare_title("The !!! 352") == "three_hundred_fifty_two"


def test_able_to_poll():
    assert utils.able_to_poll("http://www.google.com") == True
    ## It hasn't been at least two seconds since the last poll
    ## so it should fail
    assert utils.able_to_poll("http://www.google.com") == False
    time.sleep(2)
    assert utils.able_to_poll("http://www.google.com") == True
    assert utils.able_to_poll("http://www.google.com", wait=True) == True



# -*- coding: utf-8 -*-
#!/usr/bin/env python
__author__ = 'Dan Tracy'
__email__ = 'djt5019 at gmail dot com'

import sqlite3

from nose.tools import assert_raises, assert_equal
from nose.tools import nottest

from episode_renamer.Cache import Cache
from episode_renamer.Settings import Settings


@nottest
class MockEpisode(object):
    def __init__(self, name, season, number):
        self.title = name
        self.season = season
        self.episode_number = number


@nottest
class MockSpecial(object):
    def __init__(self, name, num):
        self.title = name
        self.num = num
        self.type = "Movie"


@nottest
def make_series():
    eps = []
    spc = []
    for i in xrange(100):
        name = "Episode {}".format(i)  # single ascii letter name
        s_name = "Special {}".format(i)
        season = 1
        number = i

        eps.append(MockEpisode(name, season, number))
        spc.append(MockSpecial(s_name, number))

    return eps, spc


def test_good_connection():
    cache = Cache()
    assert cache is not None
    assert cache.connection is not None
    assert cache.cursor is not None

    cache.close()

    try:
        cache.cursor.execute("SELECT * FROM table")
        assert False
    except:
        assert True

    cache = Cache(":memory:")
    cache.recreate_cache()
    assert cache is not None
    assert cache.connection is not None
    assert cache.cursor is not None

    cache.close()


def test_bad_connection():
    assert_raises(ValueError, Cache, dbName=None)
    assert_raises(ValueError, Cache, dbName=1)
    assert_raises(ValueError, Cache, dbName=False)
    assert_raises(ValueError, Cache, dbName=True)
    assert_raises(ValueError, Cache, dbName=[1, 2, 3])
    assert_raises(sqlite3.OperationalError, Cache, dbName='../')


def test_update_old_entry():
    cache = Cache(":memory:")
    cache.recreate_cache()

    eps, spc = make_series()

    cache.add_show("test show", eps)
    cache.add_specials("test show", spc)

    old_val = Settings['db_update']
    Settings['db_update'] = 0

    eps = cache.get_episodes("test show")
    spc = cache.get_specials("test show")

    Settings['db_update'] = old_val

    cache.close()


def test_add_remove_eps():
    cache = Cache(":memory:")
    cache.recreate_cache()

    eps, spc = make_series()

    Settings['db_update'] = 7

    cache.add_show("test show", eps)
    cache.add_specials("test show", spc)

    eps = cache.get_episodes("test show")
    spc = cache.get_specials("test show")

    assert len(eps) == len(spc)

    for count, entry in enumerate(zip(eps, spc)):
        e, s = entry
        name = "Episode {}".format(count)  # single ascii letter name
        s_name = "Special {}".format(count)

        assert e.title == name
        assert e.episode_number == count
        assert e.season == 1

        assert s.title == s_name
        assert s.num == count

    cache.remove_show(1)
    cache.close()


def test_remove_fake_show():
    cache = Cache(":memory:")
    cache.recreate_cache()

    assert_raises(ValueError, cache.remove_show, sid=None)
    assert_raises(ValueError, cache.remove_show, sid=[])
    assert_raises(ValueError, cache.remove_show, sid={})
    assert_raises(ValueError, cache.remove_show, sid="")

    cache.close()


def test_bad_add_show_eps():
    cache = Cache(":memory:")
    cache.recreate_cache()

    eps, spc = make_series()

    assert_raises(ValueError, cache.add_show, showTitle="", episodes=eps)
    assert_raises(ValueError, cache.add_show, showTitle=None, episodes=None)
    assert_raises(ValueError, cache.add_show, showTitle="Show", episodes=None)
    assert_raises(ValueError, cache.add_show, showTitle="Show", episodes="Title")
    assert_raises(ValueError, cache.add_show, showTitle="Title", episodes=[])
    assert_raises(ValueError, cache.add_show, showTitle="Title", episodes={})
    assert_raises(ValueError, cache.add_show, showTitle="test", episodes=[])

    cache.close()


def test_bad_add_special_eps():
    cache = Cache(':memory:')

    eps, spc = make_series()

    assert_raises(ValueError, cache.add_specials, showTitle="", episodes=spc)
    assert_raises(ValueError, cache.add_specials, showTitle=None, episodes=None)
    assert_raises(ValueError, cache.add_specials, showTitle="Show", episodes=None)
    assert_raises(ValueError, cache.add_specials, showTitle="Show", episodes="Title")
    assert_raises(ValueError, cache.add_specials, showTitle="Title", episodes=[])
    assert_raises(ValueError, cache.add_specials, showTitle="Title", episodes={})
    assert_raises(ValueError, cache.add_specials, showTitle="test", episodes=[])

    cache.close()


def test_bad_get_episodes():
    cache = Cache(':memory:')

    assert_raises(ValueError, cache.get_episodes, showTitle=None)
    assert_raises(ValueError, cache.get_episodes, showTitle={})
    assert_raises(ValueError, cache.get_episodes, showTitle=[])
    assert_equal(cache.get_episodes(showTitle="FAKE"), [])
    assert_equal(cache.get_episodes(showTitle="test"), [])

    cache.close()


def test_bad_get_specials():
    cache = Cache(':memory:')

    assert_raises(ValueError, cache.get_specials, showTitle=None)
    assert_raises(ValueError, cache.get_specials, showTitle={})
    assert_raises(ValueError, cache.get_specials, showTitle=[])
    assert_equal(cache.get_specials("test"), [])
    assert_equal(cache.get_specials("FAKE"), [])

    cache.close()



# -*- coding: utf-8 -*-
__author__ = 'Dan Tracy'
__email__ = 'djt5019 at gmail dot com'

import zipfile
import logging

from urllib import quote_plus

from eplist import utils

from eplist.episode import Episode, Special
from eplist.settings import Settings

try:
    from BeautifulSoup import BeautifulStoneSoup as Soup
except ImportError:
    logging.critical(u"Error: BeautifulSoup was not found, unable to parse theTVdb")

priority = 1


def poll(title):
    try:
        API_KEY = Settings['tvdb_key']
    except KeyError as e:
        logging.warn("The TvDB Api key was not found, unable to poll the TvDB")
        logging.warn(e)
        return utils.show_not_found

    try:
        Soup
    except NameError:
        return utils.show_not_found

    cleanTitle = quote_plus(title)

    #1) First we need to find the series ID
    seriesIdLoc = "http://www.thetvdb.com/api/GetSeries.php?seriesname={0}".format(cleanTitle)
    seriesFileDesc = utils.get_url_descriptor(seriesIdLoc)

    if seriesFileDesc is None:
        return utils.show_not_found

    seriesIdXml = Soup(seriesFileDesc.content, convertEntities=Soup.HTML_ENTITIES)

    seriesIds = seriesIdXml.findAll('series')

    if not seriesIds:
        return utils.show_not_found

    ## TODO: Handle the series conflicts in a sane manner
    if len(seriesIds) > 1:
        logging.warn("Conflict with series title ID on TVdB")
        for seriesName in seriesIdXml.findAll('seriesname'):
            logging.info("Alternate series: {}".format(seriesName.getText()))

    seriesID = seriesIds[0].seriesid.getString()
    seriesIdXml.close()

    #2) Get base info zip file
    infoLoc = "http://www.thetvdb.com/api/{0}/series/{1}/all/en.zip".format(API_KEY, seriesID)
    infoFileDesc = utils.get_url_descriptor(infoLoc)
    if infoFileDesc is None:
        return utils.show_not_found

    tempZip = utils.temporary_file(suffix='.zip')
    tempZip.seek(0)
    tempZip.write(infoFileDesc.content)

    with zipfile.ZipFile(tempZip) as z:
        if 'en.xml' not in z.namelist():
            logging.error("English episode list was not found")
            return utils.show_not_found

        with z.open('en.xml') as d:
            soup = Soup(d, convertEntities=Soup.HTML_ENTITIES)

    #3) Now we have the xml data in the soup variable, just populate the list
    count = 1
    eps = []

    for data in soup.findAll('episode'):
        name = data.episodename.getText()
        season = int(data.seasonnumber.getText())
        num = int(data.episodenumber.getText())

        if name == "":
            logging.info("The name pulled from TvDB appears to be empty")
            continue

        if 'commentary' in name.lower():
            continue

        if int(season) > 0:
            eps.append(Episode(name, num, season, count))
        else:
            eps.append(Special(name, num, 'OVA'))

        count += 1

    soup.close()
    tempZip.close()

    return eps



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

__author__ = 'Dan Tracy'
__email__ = 'djt5019 at gmail dot com'

import logging

from os.path import realpath
from sys import getdefaultencoding

config = {
    ## Filter to output episodes, specials, or both (both is default)
    'filter': 'both',

    ## Your TvDB api key, required to poll their website
    ## grab on at http://thetvdb.com/?tab=apiregister
    'tvdb_key': None,

    # Logger output files
    'log_file': 'output.log',

    ## Database file for our episodes
    'db_name': 'episodes.db',

    ## Days to wait to update the show within the database
    'db_update': 7,

    ## Where to store the old filenames from the last rename operation
    'rename_backup': 'last_rename.json',

    ## File to store the access times data
    'access_time_file': 'last_access.json',
    'access_dict': {},

    ## Time in seconds between polling a website, recommended is 2
    'poll_delay': 2,

    ## AniDB flat file with the ids of the shows visit link below for updated version from time to time
    ## http://anidb.net/api/animetitles.dat.gz
    'anidb_username': None,
    'anidb_password': None,
    'anidb_db_file': 'animetitles.dat',
    'anidb_db_url': 'http://anidb.net/api/animetitles.dat.gz',
    'anidb_http_api': 'http://api.anidb.net:9001/httpapi?request=anime&client=eprenamer&clientver=1&protover=1&aid={}',

    ## Verbose output
    'verbose': False,

    ## System encoding, windows command line can't handle utf-8
    'encoding': getdefaultencoding(),

    ## Default Format string
    'format': "<series> - <type> <count:pad> - <title>",

    ## Tag options
    'tag_config': 'tags.cfg',
    'tag_start': '<',
    'tag_end': '>',
    'tags': ['episode_name_tags', 'episode_number_tags', 'episode_count_tags', 'series_name_tags', 'season_number_tags', 'hash_tags', 'type_tags'],

    'episode_name_tags': ['name', 'episode', 'title'],
    'episode_number_tags': ['epnum', 'number', 'num'],
    'episode_count_tags': ['count', 'ep'],
    'series_name_tags': ['series', 'show'],
    'season_number_tags': ['season', 'seasons'],
    'hash_tags': ['hash', 'crc32', 'checksum'],
    'type_tags': ['type', 'format'],

    ## The current working directory
    'path': realpath('.'),

    ## Database creation schema file name
    'sql': 'schema.sql'

    }


class SettingsDict(dict):
    def __init__(self, *args, **kwargs):
        super(SettingsDict, self).__init__(*args, **kwargs)

    def __getitem__(self, item):
        val = dict.get(self, item)

        if val is None:
            logging.error("Setting {} not present in config file".format(item))
            raise KeyError("Setting: {} not present in config file".format(item))
        else:
            return val

Settings = SettingsDict(config)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

__author__ = 'Dan Tracy'
__email__ = 'djt5019 at gmail dot com'

# This command line program will take a T.V. show as input and
# will return information about each episode, such as the title
# season and number.  I use this program to help clean up my
# TV show collection.  After the show has been found online it
# will be entered into a local database, provided sqlite3 is available,
# for faster lookup in the future.  You are able to filter the shows
# by season along with other options on the command line interface.
# You can also rename files according to a format that you choose as
# well as calculate the CRC32 of a file.  If you mistakenly rename
# files you have the option to revert the last renaming operation

import argparse
import os
import sys
import logging

from eplist import utils
from eplist import episode

from eplist.logger import init_logging()
from eplist.cache import Cache
from eplist.show_finder import Parser
from eplist.settings import Settings

init_logging()

if not os.path.exists(constants.RESOURCE_PATH):
    utils.initalize_resource_folder()


def main():
    cmd = argparse.ArgumentParser(description="Renames your TV shows",
                        prog='eplist', usage='%(prog)s --help [options] title')

    cmd.add_argument('title', default="", nargs='?',
        help="The title of the show")

    cmd.add_argument('-d', '--display-header', action="store_true",
        help="Display the header at the top of the output")

    cmd.add_argument('-v', '--verbose', action="store_true",
        help="Be verbose, enable additional output")

    cmd.add_argument('-s', '--season', default="", type=str, metavar='N',
        help="The specific season range to search for. Ex: 1-3")

    cmd.add_argument('-e', '--episode', default="", type=str, metavar='N',
        help="The specific episode range to search for Ex: 15-30")

    cmd.add_argument('-f', '--format', dest="format", metavar='F',
        help="Rename the files in a directory with a custom format")

    cmd.add_argument('-g', '--gui-enabled', action="store_true",
        help="Use the gui rather than the command line")

    group = cmd.add_mutually_exclusive_group()

    group.add_argument('-r', '--rename', dest='pathname', metavar="PATH",
        help="Rename the files in the path provided")

    group.add_argument('-u', '--undo-rename', action='store_true',
        help="Undo the last rename operation")

    cmd.add_argument('--delete-cache', action="store_true",
        help="Delete the cache file and create a new one")

    cmd.add_argument('--update-db', action="store_true",
        help="Update the AniDB titles file, limit to once a day due to size")

    cmd.add_argument('--verify', action="store_true",
        help="Verify the checksums in the filename if they are present")

    cmd.add_argument('--filter', choices=['episodes', 'specials', 'both'],
        help="Filters episodes based on type (default=both)")

    args = cmd.parse_args()

    if 'title' in args:
        Settings['title'] = args.title
    else:
        Settings['title'] = ""

    # Set the correct working path
    if args.pathname:
        Settings['path'] = os.path.realpath(args.pathname)
    else:
        Settings['path'] = os.path.realpath(os.getcwd())

    # If we are acting on files then load the old names into memory
    if args.pathname or args.undo_rename:
        utils.load_renamed_file()

    if args.filter:
        Settings['filter'] = args.filter

    cache = Cache(Settings['db_name'])
    if args.delete_cache:
        cache.recreate_cache()

    if args.title in ('-', '.', 'pwd'):
        # If a dash is entered use the current basename of the path
        args.title = os.path.split(os.getcwd())[1]
        print ("Searching for {}".format(args.title))

    if args.verbose:
        Settings['verbose'] = True
        l = logging.getLogger()
        for handle in l.handlers:
            handle.setLevel(logging.NOTSET)
        l.setLevel(logging.NOTSET)

    if args.gui_enabled:
        import episode_renamer.gui.gui as gui
        exit(gui.main())

    if args.update_db:
        utils.update_db()

    if args.undo_rename:
        file_dict = utils.find_old_filenames(Settings['path'], args.title)
        files = file_dict['file_list']
        print_renamed_files(files)
        old_order, errors = utils.rename(files)

        utils.save_renamed_file_info(old_order, Settings['title'])

        for e in errors:
            print("File {} could not be successfully renamed".format(os.path.split(e)[1]))

        sys.exit(0)

    rename = args.pathname is not None

    if rename and not os.path.exists(args.pathname):
        sys.exit("ERROR - Path provided does not exist")

    if not args.title:
        cmd.print_usage()
        sys.exit(1)

    episodeParser = Parser(args.title, cache)

    show = episodeParser.getShow()
    formatter = episode.EpisodeFormatter(show, args.format)
    show.formatter = formatter

    if not show.episodes:
        sys.exit(1)

    # If the user specified a specific season we will filter our results
    # this also checks to make sure its a reasonable season number
    filtered_episodes = []
    if args.season:
        s_range = list(parse_range(args.season))
        if s_range[-1] > show.num_seasons:
            print ("{} Season {} not found".format(args.title, args.season))
            sys.exit(1)
        filtered_episodes = [x for x in show.episodes if x.season in s_range]

    if args.episode:
        e_range = list(parse_range(args.episode))

        if not args.season:
            filtered_episodes = [x for x in show.episodes if x.episode_count in e_range]
        else:
            filtered_episodes = filtered_episodes[e_range[0] - 1:e_range[-1]]

    ## Renaming functionality
    if  rename:
        path = args.pathname if args.pathname != '.' else os.getcwd()
        utils.prepare_filenames(path, show)
        files = []

        if filtered_episodes:
            episodes = filtered_episodes
        else:
            episodes = show.episodes

        episodes += show.specials if Settings['filter'] in ('both', 'specials') else []

        for e in episodes:
            if e.episode_file and e.episode_file.new_name:
                old = os.path.join(path, e.episode_file.name)
                new = os.path.join(path, e.episode_file.new_name)
                files.append((old, new))

        print_renamed_files(files)
        old_order, errors = utils.rename(files)

        utils.save_renamed_file_info(old_order, Settings['title'])

        if not errors:
            print ("All files were successfully renamed")

        for e in errors:
            print("File {} could not be successfully renamed".format(os.path.split(e)[1]))
        sys.exit(0)

    print ""

    if args.verify:
        verify_files(show)
        exit(1)

    if Settings['filter'] in ('both', 'episodes'):
        if filtered_episodes:
            display_episodes(show, filtered_episodes, args.display_header)
        else:
            display_episodes(show, show.episodes, args.display_header)

    if Settings['filter'] in ('specials', 'both'):
        display_specials(show, args.display_header)


def display_episodes(show, episodes, header=False):
    if header:
        print ("\nShow: {0}".format(show.title))
        print ("Number of episodes: {0}".format(len(show.episodes)))
        print ("Number of specials: {0}".format(len(show.specials)))
        print ("Number of seasons: {0}".format(show.episodes[-1].season))
        print ("-" * 30)

    curr_season = episodes[0].season
    for eps in episodes:
        if curr_season != eps.season and header:
            print ("\nSeason {0}".format(eps.season))
            print ("----------")

        print (show.formatter.display(eps).encode(Settings['encoding'], 'ignore'))
        curr_season = eps.season


def display_specials(show, header=False):
    if header:
        print ("\nSpecials")
        print ("---------")

    for eps in show.specials:
        print (show.formatter.display(eps).encode(Settings['encoding'], 'ignore'))


def verify_files(show):
    if not show:
        return()

    episode_files = show.episodes
    path = Settings.get('path', os.getcwd())
    if not all([e.episode_file for e in episode_files]):
        utils.prepare_filenames(path, show)

    for f in episode_files:
        if not f.episode_file:
            continue

        ep_file = f.episode_file

        if ep_file.given_checksum <= 0:
            print("Episode {} dosen't have a checksum to compare to".format(ep_file.name))
            continue

        if ep_file.verify_integrity():
            print("Episode {} has passed verification".format(ep_file.name))
        else:
            print("Episode {} has failed verification".format(ep_file.name))


def print_renamed_files(files):
    if not files:
        print ("Failed to find any files to rename")
        sys.exit(1)

    p = os.path.dirname(files[0][0])
    print ("PATH = {}".format(p))
    print ("-------" + '-' * len(p))

    for old, new in files:
        print (u"OLD: {0}".format(os.path.split(old)[1]).encode(Settings['encoding'], 'ignore'))
        print (u"NEW: {0}".format(os.path.split(new)[1]).encode(Settings['encoding'], 'ignore'))
        print


def parse_range(range):
    if '-' in range:
        high, low = range.split('-')
        high = int(high)
        low = int(low)
    else:
        high = low = int(range)

    low = max(low, 1)
    high = max(high, 1)

    if low > high:
        low, high = high, low

    return xrange(low, high + 1)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

__author__ = 'Dan Tracy'
__email__ = 'djt5019 at gmail dot com'

import re
import os
import zlib
import string
import logging

import utils

from settings import Settings


class Show(object):
    """
    A convenience class to keep track of the list of episodes as well as
    to keep track of the custom formatter for those episodes
    """
    def __init__(self, seriesTitle):
        self.title = utils.encode(string.capwords(seriesTitle))
        self.proper_title = utils.prepare_title(seriesTitle.lower())
        self._episodeList = []
        self._episodes_by_season = {}
        self.specials = []
        self._formatter = None
        self.num_seasons = 0
        self.max_episode_number = 0

    @property
    def episodes(self):
        return self._episodeList

    @episodes.setter
    def episodes(self, eps):
        """
        Add episodes to the shows episode list
        """
        if eps:
            self._episodeList = eps
            self.num_seasons = eps[-1].season
            self.max_episode_number = max(x.episode_number for x in eps)
            self.num_episodes = len(eps)

            for ep in self._episodeList:
                self._episodes_by_season.setdefault(ep.season, []).append(ep)

    @property
    def formatter(self):
        if self._formatter:
            return self._formatter
        else:
            raise AttributeError("Formatter not attached to this show '{}'".format(self.show_title))

    @formatter.setter
    def formatter(self, fmt=None):
        if not fmt or not isinstance(fmt, EpisodeFormatter):
            raise ValueError("Expected a formatter but got {}".format(type(fmt)))

        self._formatter = fmt
        self._formatter.show = self

    @property
    def show_title(self):
        return self.title

    @show_title.setter
    def show_title(self, val):
        """
        Sets the shows title to the value passed as well as prepares it for use
        """
        logging.debug("Setting show title to: {}".format(val))
        self.title = utils.encode(val.capitalize())
        self.proper_title = utils.prepare_title(val)

    def get_season(self, season):
        """
        Returns a list of episodes within the season or an empty list
        """
        return self._episodes_by_season.get(season, [])

    def get_episode(self, episode, season=0):
        """
        Returns a specific episode from a specific season or None
        """
        if not 0 < episode < len(self.episodes) + 1:
            return None

        # Adjust by one since episodes start count at 1 not 0
        episode -= 1

        if season > 0:
            season = self._episodes_by_seasonSeason.get(season, None)
            if season:
                return season[episode]
            else:
                return None
        else:
            return self.episodes[episode]

    def get_special(self, special_number):
        if 0 < special_number < len(self.specials) + 1:
            return self.specials[special_number - 1]
        return None


class Episode(object):
    """
    A simple class to organize the episodes
    """
    def __init__(self, episode_title, episode_number, season, episode_count):
        """
        A container for an episode's information collected from the web
        """
        self.title = utils.encode(episode_title)
        self.season = int(season)
        self.episode_number = int(episode_number)
        self.episode_count = int(episode_count)
        self.episode_file = None
        self.type = "Episode"


class Special(object):
    """
    Container class for Specials/Movies/OVAs
    """
    def __init__(self, special_title, special_number, special_type):
        self.title = special_title
        self.num = int(special_number)
        self.type = special_type
        self.episode_file = None


class EpisodeFile(object):
    """
    Represents a TV show file.  Used for renaming purposes
    """
    def __init__(self, path, **kwargs):
        """
        A physical episode on disk
        """
        self.path = path
        self.ext = os.path.splitext(self.path)[1]
        self.name = utils.encode(os.path.split(self.path)[1])
        self.new_name = ""
        self.verified = False
        self.__dict__.update(kwargs)
        self.is_ova = ('special_number' in kwargs)
        self.given_checksum = kwargs.get('checksum', 0)
        self.episode_number = self.__dict__.get('episode_number', -1)

    def crc32(self):
        """
        Calculate the CRC32 checksum for a file, painfully slow
        """
        logging.info("calculating CRC for {}".format(os.path.split(self.path)[1]))
        with open(self.path, 'rb') as f:
            checksum = 0
            for line in f:
                checksum = zlib.crc32(line, checksum)

        return checksum & 0xFFFFFFFF

    def verify_integrity(self):
        """
        Compares the checksum in the filename to the calculated one
        """
        if self.verified:
            return True

        if self.given_checksum > 0:
            if self.given_checksum == self.crc32():
                self.verified = True
                return True
        return False


class EpisodeFormatter(object):
    def __init__(self, show, fmt=None):
        """
        Allows printing of custom formatted episode information
        """
        self.show_ = show
        self._format_string = utils.encode(fmt) if fmt else Settings['format']

        ## Use negative lookahead assertion to ensure that the tag had not been escaped
        re_format = r'(?<!\\)(?P<tag>\{start}.*?\{end})'
        re_format = re_format.format(start=Settings['tag_start'], end=Settings['tag_end'])

        self.re = re.compile(re_format, re.I)
        self.tokens = self.re.split(self._format_string)
        self.strip_whitespace_regex = re.compile(r'[\s]+')

        self.episode_number_tags = None
        self.type_tags = None
        self.season_number_tags = None
        self.episode_count_tags = None
        self.episode_name_tags = None
        self.hash_tags = None
        self.series_name_tags = None
        self.tags = filter(lambda x: x.endswith('tags'), dir(self))
        self.settings = dict(upper=False, lower=False, pad=False, proper=False)
        self.load_format_config()

    @property
    def show(self):
        return self.show_

    @show.setter
    def show(self, show=None):
        if not show:
            raise AttributeError("Expected a Show object but got {}".format(type(show)))
        logging.debug("Setting show to {}".format(show.title))
        self.show_ = show

    @property
    def format_string(self):
        return self._format_string

    @format_string.setter
    def format_string(self, fmt=None):
        """
        Set the format string for the formatter
        """
        if fmt is not None:
            self._format_string = utils.encode(fmt)
            self.tokens = self.re.split(fmt)
        else:
            raise AttributeError("Empty format string set")

    def load_format_config(self):
        """
        Load tokens from the format setting in settings.py
        """
        allTokens = set()
        for s in self.tags:
            tokens = set(Settings[s])

            redefined = tokens.intersection(allTokens)
            if redefined:
                #Look for duplicates
                msg = "In section [{}]: tokens '{}' redefined".format(s, map(str, redefined))
                logging.error(msg)
                raise AttributeError(msg)

            allTokens = allTokens.union(tokens)

            self.__dict__[s] = tokens

    def display(self, ep):
        """
        Displays the episode according to the users format
        """
        args = []
        escaped_token = "\{}".format(Settings['tag_start'])
        for token in self.tokens:
            if escaped_token in token:
                args.append(token.replace(escaped_token, Settings['tag_start']))
            elif self.re.match(token):
                #If it's a tag try to resolve it
                token = self.strip_whitespace_regex.sub("", token)
                args.append(self._parse(ep, token[1:-1]))
            else:
                args.append(token)

        return utils.encode(''.join(args))

    def _parse_modifiers(self, tag):
        # Tag modifiers such as number padding and caps
        if ':' in tag:
            res = re.split(':', tag)
            tag = res[0]
            modifiers = res[1:]
        else:
            return tag

        if 'pad' in modifiers:
            self.settings['pad'] = True

        if 'caps' in modifiers or 'upper' in modifiers:
            self.settings['upper'] = True
        elif 'lower' in modifiers:
            self.settings['lower'] = True
        elif 'proper' in modifiers:
            self.settings['proper'] = True

        return tag

    def _parse(self, ep, tag):
        """
        Tokenize and substitute tags for their values using an episode
        as well as the episode file for reference
        """
        self.settings['proper'] = False
        self.settings['upper'] = False
        self.settings['lower'] = False
        self.settings['pad'] = False

        tag = self._parse_modifiers(tag.lower())

        if tag in self.episode_number_tags:
            return self._handle_episode_number(ep)

        elif tag in self.type_tags:
            return self._handle_string(ep.type)

        elif tag in self.season_number_tags:
            return self._handle_season(ep)

        elif tag in self.episode_count_tags:
            return self._handle_episode_counter(ep)

        elif tag in self.episode_name_tags:
            return self._handle_string(ep.title)

        elif tag in self.series_name_tags:
            return self._handle_string(self.show.title)

        elif tag in self.hash_tags:
            return self._handle_hash(ep)

        else:
            # If it reaches this case it's most likely an invalid tag
            return Settings['tag_start'] + tag + Settings['tag_end']

    def _handle_string(self, string_):
        if self.settings['lower']:
            return string_.lower()
        elif self.settings['upper']:
            return string_.upper()
        elif self.settings['proper']:
            return string.capwords(string_)
        else:
            return string_

    def _handle_number(self, number, pad_length):
        if self.settings['pad']:
            return str(number).zfill(pad_length)
        else:
            return str(number)

    def _handle_season(self, ep):
        if isinstance(ep, Special):
            # Going on the basis that specials don't have seasons
            return ""

        pad = len(str(self.show.num_seasons))
        return self._handle_number(ep.season, pad)

    def _handle_episode_number(self, ep):
        if isinstance(ep, Episode):
            number = ep.episode_number
            pad = len(str(self.show.max_episode_number))
        else:
            number = ep.num
            pad = len(str(self.show.specials[-1].num))

        return self._handle_number(number, pad)

    def _handle_episode_counter(self, ep):
        if isinstance(ep, Episode):
            number = ep.episode_count
            pad = len(str(self.show.num_episodes))
        else:
            number = ep.num
            pad = len(str(self.show.specials[-1].num))

        return self._handle_number(number, pad)

    def _handle_hash(self, ep):
        if not ep.episode_file:
            return "00000000"

        if ep.episode_file.checksum > 0:
            return self._handle_string(hex(ep.episode_file.checksum))

        else:
            # To remove the 0x from the hex string
            return self._handle_string(hex(ep.episode_file.crc32())[2:10])



# -*- coding: utf-8 -*-
__author__ = 'Dan Tracy'
__email__ = 'djt5019 at gmail dot com'

from episode_renamer import Parser

from nose.tools import nottest


@nottest
class MockCache(object):
    """docstring for MockCache"""
    def __init__(self, arg):
        super(MockCache, self).__init__()
        self.arg = arg

    def add_specials(self, *args, **kwargs):
        return True

    def add_episodes(self, *args, **kwargs):
        return True

    def get_episodes(self, *args, **kwargs):
        return None

    def get_specials(self, *args, **kwargs):
        return None


@nottest
class MockPollSources(object):
    """docstring for MockPollSources"""
    def __init__(self, arg):
        super(MockPollSources, self).__init__()
        self.arg = arg

    def locate_show(self):
        return []


def test_get_show():
    parser = Parser()



# -*- coding: utf-8 -*-
__author__ = 'Dan Tracy'
__email__ = 'djt5019 at gmail dot com'

from eplist import utils
from eplist.episode import Episode

priority = 2

pattern = r"""
            ^		                # Start of the string
            (?:[\s]*?[\d]*\.?)	    # Number on list
            [\s]{2,}		        # Ignore whitespace
            (?P<season>\d+)  	    # Season number
            -			            # Separator
            [\s]*		            # Optional whitespace
            (?P<episode>\d+)	    # Episode number
            [\s]{2,}		        # Whitespace
            (?P<product>.+|)	    # Product number
            [\s]{1,}		        # Whitespace
            (?P<airdate>[\w\s/]*?)  # Air-date
            [\s]{2,}		        # Ignore whitespace
            (?P<name>.*)	        # Episode name
            $			            # End of line
            """


def poll(title):
    cleanTitle = utils.prepare_title(title)
    episodes = []
    url = "http://www.epguides.com/{0}".format(cleanTitle)
    fd = utils.get_url_descriptor(url)

    if fd is None:
        return utils.show_not_found

    regex = utils.regex_compile(pattern)

    count = 1
    for line in fd.iter_lines():
        info = regex.match(line)
        if info is not None:
            name = info.group('name')
            episode = info.group('episode')
            season = int(info.group('season'))
            name = utils.regex_sub('<.*?>', '', name).strip()

            if '[Trailer]' in name:
                name = name.replace('[Trailer]', '')

            if name == "TBA":
                continue

            episodes.append(Episode(name, episode, season, count))
            count += 1

    return episodes



# -*- coding: utf-8 -*-
__author__ = 'Dan Tracy'
__email__ = 'djt5019 at gmail dot com'

import atexit
import logger
logger.init_logging()

atexit.register(utils.save_last_access_times)
atexit.register(logger.shutdown_logging)

utils.load_renamed_file()



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

__author__ = 'Dan Tracy'
__email__ = 'djt5019 at gmail dot com'

import re

import os
import sys
from os.path import join, split, realpath

VIDEO_EXTENSIONS = set(['.mkv', '.ogm', '.asf', '.asx', '.avi', '.flv', '.mov', '.mp4', '.mpg', '.rm', '.swf', '.vob',
                    '.wmv', '.mpeg'])

PROJECT_SOURCE_PATH = split(realpath(__file__))[0]
PROJECT_PATH = split(PROJECT_SOURCE_PATH)[0]
WEB_SOURCES_PATH = join(PROJECT_SOURCE_PATH, 'web_sources')

RESOURCE_PATH = os.path.join("eplist", "resources")

if sys.platform == "win32":
    RESOURCE_PATH = os.path.join(os.environ['APPDATA'], RESOURCE_PATH)
elif sys.platform == 'darwin':
    ## Found at http://stackoverflow.com/questions/1084697/how-do-i-store-desktop-application-data-in-a-cross-platform-way-for-python
    from AppKit import NSSearchPathForDirectoriesInDomains
    RESOURCE_PATH = os.path.join(NSSearchPathForDirectoriesInDomains(14, 1, True)[0], RESOURCE_PATH)
else:  # *nix / solaris
    RESOURCE_PATH = os.path.expanduser(os.path.join("~", "." + RESOURCE_PATH))


SHOW_NOT_FOUND = []

regexList = []

NUM_DICT = {'0': '', '1': 'one', '2': 'two', '3': 'three', '4': 'four', '5': 'five', '6': 'six',
        '7': 'seven', '8': 'eight', '9': 'nine', '10': 'ten', '11': 'eleven', '12': 'twelve',
        '13': 'thirteen', '14': 'fourteen', '15': 'fifteen', '16': 'sixteen', '17': 'seventeen',
        '18': 'eighteen', '19': 'nineteen', '20': 'twenty', '30': 'thirty', '40': 'forty',
        '50': 'fifty', '60': 'sixty', '70': 'seventy', '80': 'eighty', '90': 'ninety'}


regex_vars = {
'sep': r'[\-\~\.\_\s]',
'sum': r'(.*[\[\(](?P<sum>[a-z0-9]{8})[\]\)])',
'year': r'(:P<year>(19|20)?\d\d)',
'episode': r'(e|ep|episode)?{sep}*?(?P<episode>\d+)(?:v\d)?',  # ex: e3v2
'season': r'(s|season)?{sep}*?(?P<season>\d+)',
'series': r'(?P<series>.*)',
'subgroup': r'(?P<group>\[.*\])',
'special': r'(?P<type>ova|ona|extra|special|movie|dvd|bluray){sep}+(?P<special>\d+)',
}

for k, v in regex_vars.iteritems():
    try:
        # Substitute any regex variables that may have been used within later dictionary entries
        regex_vars[k] = v.format(**regex_vars)
    except IndexError as e:
        pass

uncompiled_regex = [
            r'^(?P<series>.*?) - Season (?P<season>\d+) - Episode (?P<episode>\d*) - .*',  # Also mine
            r'^(?P<series>.*?) - Episode (?P<episode>\d*) - .*',  # My usual format
            r'^{series}{sep}+{special}',
            r'^{series}{sep}+{episode}',
            r'^{series}{sep}+{season}{sep}*{episode}',
            r'^{series}{sep}+{season}{sep}*{episode}{sep}*{sum}?',
            r'^{series}{sep}+{episode}',
            r'^(?P<series>.*) - OVA (?P<special>\d+) - \w*',
            r'^{series}{sep}*{special}',
            r'{series}{sep}*(op|ed){sep}*(?P<junk>\d*)',  # Show intro /outro music, just ignore them
            r'.*{episode}',  # More of a general catch-all regex, last resort search for the first numbers in the filename
            ]

## Substitute the dictionary variables in to the unformated regexes (is the plural of regex, regexes?)
uncompiled_regex = [r.format(**regex_vars) for r in uncompiled_regex]

regexList = map(lambda x: re.compile(x, re.I), uncompiled_regex)

checksum_regex = re.compile(r'[\[\(](?P<sum>[a-f0-9]{8})[\]\)]', re.I)
remove_junk_regex = re.compile(r'[\[\(].*?[\]\]]', re.I)
bracket_season_regex = re.compile(r'[\[\(]{season}X{episode}[\]\)]'.format(**regex_vars), re.I)

del k, e, v, regex_vars, uncompiled_regex


CREATE_DB_QUERY = """
PRAGMA foreign_keys = ON;

DROP TABLE IF EXISTS episodes;
DROP TABLE IF EXISTS specials;
DROP TABLE IF EXISTS shows;

CREATE TABLE shows (
    sid INTEGER PRIMARY KEY,
    title TEXT NOT NULL,
    time TIMESTAMP
);

CREATE TABLE episodes (
    eid INTEGER PRIMARY KEY,
    sid INTEGER NOT NULL REFERENCES shows(sid) ON DELETE CASCADE,
    eptitle TEXT NOT NULL,
    season INTEGER NOT NULL,
    showNumber INTEGER NOT NULL
);

CREATE TABLE specials(
    mid INTEGER PRIMARY KEY,
    sid INTEGER NOT NULL REFERENCES shows(sid) ON DELETE CASCADE,
    title TEXT NOT NULL,
    showNumber INTEGER NOT NULL,
    type TEXT NOT NULL
);
"""



# -*- coding: utf-8 -*-
__author__ = 'Dan Tracy'
__email__ = 'djt5019 at gmail dot com'

import unittest

from episode_renamer.web_sources import anidb


class TestAniDB(unittest.TestCase):
    def test_local_file(self):
        assert(anidb._parse_local("FAKE SHOW") < 0)
        assert(anidb._parse_local("black lagoon") == anidb._parse_local("BLACK LAGOON"))
        assert(anidb._parse_local("clannad") != anidb._parse_local("black lagoon"))



# -*- coding: utf-8 -*-
__author__ = 'Dan Tracy'
__email__ = 'djt5019 at gmail dot com'

import difflib
import logging

from string import punctuation as punct

from eplist import utils

from eplist.episode import Episode, Special
from eplist.settings import Settings

try:
    from BeautifulSoup import BeautifulStoneSoup as Soup
except ImportError:
    logging.critical(u"Error: BeautifulSoup was not found, unable to parse AniDB")

priority = 3


def _parse_local(title):
    """
    Try to find the anime ID (aid) in the dump file provided by AniDB
    """
    if not utils.file_exists_in_resources('animetitles.dat'):
        logging.warning("AniDB database file not found, unable to poll AniDB at this time")
        logging.warning("Try using the --update-db option to download an copy of it")
        return -1

    title = title.lower()

    regex = utils.regex_compile(r'(?P<aid>\d+)\|(?P<type>\d)\|(?P<lang>.+)\|(?P<title>.*)')

    sequence = difflib.SequenceMatcher(lambda x: x in punct, title.lower())

    guesses = []

    with utils.open_file_in_resources('animetitles.dat') as f:
        for line in f:
            res = regex.search(line)

            if not res:
                continue

            original_title = utils.encode(res.group('title').lower())
            clean_title = utils.remove_punctuation(utils.encode(res.group('title'))).lower()

            if title in (original_title, clean_title):
                return res.group('aid')

            sequence.set_seq2(clean_title.lower())
            ratio = sequence.ratio()

            if ratio > .80:
                d = dict(ratio=ratio, aid=res.group('aid'), title=original_title)
                guesses.append(d)

    if guesses:
        print guesses
        logging.info("{} possibilities".format(len(guesses)))
        guesses = sorted(guesses, key=lambda x: x['ratio'])
        _, aid, name = guesses[0]
        logging.error("Closest show to '{}' is {} with id {}".format(title, name, aid))

        for guess in guesses[1:]:
            logging.info("Similar show {} [{}] also found".format(name, aid))

    return -1


def _connect_UDP(aid, interactive=True):
    ## Todo: make this work so we stop relying on the http protocol
    raise NotImplementedError("I will get to this later... promise.")

    if not Settings['anidb_username'] or not Settings['anidb_password']:
        raise ValueError("Username/Password required to poll AniDB")


def _connect_HTTP(aid):
    """
    Connect to AniDB using the public HTTP Api, this is used as an alternative to the UDP connection function
    """
    url = Settings['anidb_http_api'].format(aid)

    resp = utils.get_url_descriptor(url)

    if resp is None:
        return []

    soup = Soup(resp.content)

    if soup.find('error'):
        logging.error("Temporally banned from AniDB, most likely due to flooding")
        return []

    episodes = soup.findAll('episode')

    if not episodes:
        return []

    epList = []

    for e in episodes:
        # 1 is a normal episode, 2 is a special
        ep_type = e.epno.attrs[0][1]
        if ep_type not in ('1', '2'):
            continue

        title = e.find('title', {'xml:lang': 'en'})
        title = title.getText()

        if ep_type == '1':
            epNum = int(e.epno.getText())
            epList.append(Episode(utils.encode(title), epNum, 1, epNum))
        else:
            epNum = int(e.epno.getText()[1:])
            epList.append(Special(utils.encode(title), epNum, 'OVA'))

    return epList


def poll(title=""):
    try:
        Soup
    except NameError:
        return utils.show_not_found

    aid = _parse_local(title.lower())

    if aid < 0:
        return utils.show_not_found

    logging.info("Found AID: {}".format(aid))

    if utils.able_to_poll('AniDB'):
        episodes = _connect_HTTP(aid)
        if episodes:
            return episodes

    return utils.show_not_found



#!/usr/bin/env python
from distutils.core import setup
from setuptools import find_packages

setup(
    name='eplist',
    version='0.1.3',
    description='Simple episode renaming program',
    long_description=open('README.rst').read(),
    author='Dan Tracy',
    author_email='djt5019@gmail.com',
    url='https://github.com/djt5019/episode_renamer',
    packages=find_packages(),
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "Topic :: Multimedia :: Video",
        "Topic :: Utilities",
        "Environment :: Console",
        "Environment :: X11 Applications :: Qt",
        "Operating System :: OS Independent",
    ],
    requires=[
        "BeautifulSoup (>=3.2.0)",
        "requests (>=0.9.1)",
    ],
    entry_points={
        'console_scripts': ['eplist = eplist.main:main']
    },
    package_data={'': ['eplist.py', 'LICENSE', 'README.rst']},
    include_package_data=True,
)



# -*- coding: utf-8 -*-
__author__ = 'Dan Tracy'
__email__ = 'djt5019 at gmail dot com'

import logging

import poll_sources
import utils

from episode import Show


class Parser(object):
    """The main parser will poll the internet as well as a database
    looking for the show by using the parseData() function"""

    def __init__(self, showTitle="", cache=None):
        """ Proper title is used for the database/url while the display
        title is used for error messages/display purposes"""
        self.show = Show(showTitle)
        self.cache = cache

    def setShow(self, showTitle):
        """Sets a new show to search for, the old show will be removed """
        if showTitle:
            self.show = Show(showTitle)
            self.show.proper_title = utils.prepare_title(showTitle.lower())
            self.show.title = showTitle
        else:
            raise ValueError("Empty show title passed to set show")

    def getShow(self):
        """ The main driver function of this class, it will poll
        the database first, if the show doesn't exist it will
        then try the internet. """

        if self.show.title == '':
            return None

        if self.cache:
            (episodes, specials) = self._parseCacheData()
            self.show.episodes = episodes
            self.show.specials = specials

        # The show was found in the database
        if self.show.episodes:
            logging.info("Show found in database")
            return self.show

        # The show was not in the database so now we try the website
        logging.info("Show not found in database, polling web")
        eps, specials = self._parseHTMLData()
        self.show.episodes = eps
        self.show.specials = specials

        if not self.show.episodes:
            logging.error("Show was not found, check spelling and try again")
            return self.show

        # If we successfully find the show from the internet then
        # we should add it to our database for later use
        if self.cache is not None:
            logging.info("Adding show to the database")
            self.cache.add_show(self.show.proper_title, self.show.episodes)
            self.cache.add_specials(self.show.proper_title, self.show.specials)

        return self.show

    def _parseCacheData(self):
        """The query should return a positive show id otherwise
        it's not in the database"""
        eps = self.cache.get_episodes(self.show.proper_title)
        spc = self.cache.get_specials(self.show.proper_title)
        return (eps, spc)

    def _parseHTMLData(self):
        """ Passes the sites contents through the regex to seperate episode
        information such as name, episode number, and title.  After getting
        the data from the regex we will occupy the episode list """

        return poll_sources.locate_show(self.show.title)
